use clap::{Parser, Subcommand};
use clap::CommandFactory as _;
use clap::builder::StyledStr;
use clap_complete::engine::{ArgValueCompleter, CompletionCandidate};
use clap_complete::CompleteEnv;
use anyhow::Result;
use serde::Deserialize;

fn nix_eval(args: Vec<&str>) -> Result<String> {
    Ok(String::from_utf8(std::process::Command::new("nix").args(["eval", "--json"]).args(args.clone()).output()?.stdout).unwrap())
}

#[derive(Debug,Clone,Deserialize)]
enum Filter {
    Host(Host),
    Label(Label),
}

impl Into<CompletionCandidate> for Filter {
    fn into(self) -> CompletionCandidate {
        match self {
            Filter::Host(h) => {
                CompletionCandidate::new(h.name)
                    .tag(Some(StyledStr::from("host")))
            },
            Filter::Label(l) => {
                CompletionCandidate::new(l.name)
                    .tag(Some(StyledStr::from("label")))
                    .add_prefix("@")
            },
        }
    }
}
type FQDN = Vec<String>;

#[derive(Debug,Clone,Deserialize)]
struct Label {
    pub name: String,
    pub members: Option<Vec<FQDN>>,
}

#[derive(Debug,Clone,Deserialize)]
struct Host {
    pub name: String,
    pub fqdn: FQDN,
}
impl From<String> for Host {
    fn from(n:String) -> Host {
        Host{ 
            name: n.clone(), 
            fqdn: vec![n], // FIXME: add fqdn
        }
    }
}

fn filter_completer(current: &std::ffi::OsStr) -> Vec<CompletionCandidate> {
    let args = vec!["../nix-files/#nixosConfigurations", "--apply", "builtins.attrNames"]; 
    let host: Vec<String> = serde_json::from_str(nix_eval(args).unwrap().as_str()).unwrap();
    host.into_iter().map(|e| Filter::Host(e.into()).into()).collect()
}

#[derive(Debug, Parser)]
struct Cli {
    #[arg(long,short, add = ArgValueCompleter::new(filter_completer))]
    limit: Option<Vec<String>>,
    #[command(subcommand)]
    command: Commands,
}

#[derive(Debug, Subcommand)]
enum Commands {
    /// evaluates the flake and caches it
    archive,
}

fn main() {
    CompleteEnv::with_factory(|| Cli::command()).complete()
//    print!("{:?}",filter_completer(&std::ffi::OsStr::new("")))
}
